package com.xiaoliu;

import com.xiaoliu.codec.DecodeHandler;
import com.xiaoliu.codec.EncodeHandler;
import com.xiaoliu.console.ConsoleManager;
import com.xiaoliu.console.impl.SendFileConsole;
import com.xiaoliu.handler.FilePacketServerHandler;
import com.xiaoliu.handler.FileReceiveServerHandler;
import com.xiaoliu.handler.FileSendServerHandler;
import com.xiaoliu.handler.JoinClusterRequestHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import lombok.extern.slf4j.Slf4j;

import java.util.Scanner;

@Slf4j
public class Server {

    private static final int PORT = Integer.parseInt(System.getProperty("port", "8080"));

    public static void main(String[] args) throws InterruptedException {
        init();
    }

    public static void init() throws InterruptedException {
        init(PORT);
    }

    public static void init(int port) throws InterruptedException {

        if(port == 0){
            port = 8080;
        }

        log.info("服务启动开始...");

        ServerBootstrap bootstrap = new ServerBootstrap();

        EventLoopGroup boss = new NioEventLoopGroup(2);
        EventLoopGroup worker = new NioEventLoopGroup(10);

        bootstrap.group(boss, worker)
                .channel(NioServerSocketChannel.class)
                .option(ChannelOption.SO_BACKLOG, 1024)
                .option(ChannelOption.TCP_NODELAY, true)
                .childHandler(new ChannelInitializer<NioSocketChannel>() {
                    @Override
                    protected void initChannel(NioSocketChannel channel) {
                        ChannelPipeline pipeline = channel.pipeline();
                        pipeline.addLast(new FileReceiveServerHandler());
                        pipeline.addLast(new FileSendServerHandler());
                        pipeline.addLast(new DecodeHandler());
                        pipeline.addLast(new EncodeHandler());
                        pipeline.addLast(new JoinClusterRequestHandler());
                        pipeline.addLast(new FilePacketServerHandler());
                        // pipeline.addLast("handler", new MyServerHandler());
                    }
                });

        ChannelFuture future = bootstrap.bind(port).sync();
        if (future.isSuccess()) {
            log.info("端口绑定成功");
        } else {
            log.info("端口绑定失败");
        }

        future.channel().closeFuture().sync();
    }

}
